@@ -1,7 +1,7 @@
 #!/bin/bash
 
 function usage {
-    echo "./$(basename $0) {stable,wip} {bionic,focal} {create,sync,changes,build,put,clean}*"
+    echo "./$(basename $0) {stable,wip} {jammy,focal,bionic} {create,sync,changes,build,put,clean}*"
 }
 
 if [[ $# < 1 ]]; then
@@ -24,7 +24,7 @@ rep="${1}"
 shift
 
 case "${rep}" in
-    focal|bionic)
+    jammy|focal|bionic)
         ;;
     *)
         usage
@@ -52,10 +52,12 @@ for act in "${@}"; do
                 --exclude '*.egg-info' \
                 --exclude dist \
                 --exclude build \
+                --exclude third_party/catch \
                 ../../app \
                 ../../cmake \
                 ../../clasp \
                 ../../lib* \
+                ../../third_party \
                 ../../CMakeLists.txt \
                 ../../README.md \
                 ../../INSTALL.md \
@@ -81,7 +83,7 @@ EOF
                 cd "${rep}"
                 pdebuild --buildresult .. --auto-debsign --debsign-k 744d959e10f5ad73f9cf17cc1d150536980033d5 -- --basetgz /var/cache/pbuilder/${ref}-${rep}.tgz --source-only-changes
                 sed -i '/\.buildinfo$/d' ../clingo_${VERSION}_source.changes
-                debsign --no-re-sign -k744d959e10f5ad73f9cf17cc1d150536980033d5 ../clingo_${VERSION}_source.changes
+                debsign --re-sign -k744d959e10f5ad73f9cf17cc1d150536980033d5 ../clingo_${VERSION}_source.changes
             )
             ;;
         put)
@@ -94,6 +96,7 @@ EOF
                 "${rep}"/cmake \
                 "${rep}"/clasp \
                 "${rep}"/lib* \
+                "${rep}"/third_party \
                 "${rep}"/CMakeLists.txt \
                 "${rep}"/README.md \
                 "${rep}"/INSTALL.md \
