@@ -577,7 +577,8 @@ bool DefaultUnfoundedCheck::assertAtom(Literal a, UfsType t) {
 		computeReason(t);
 	}
 	activeClause_[0] = ~a;
-	bool noClause = solver_->isTrue(a) || strategy_ == no_reason || strategy_ == only_reason || (strategy_ == shared_reason && activeClause_.size() > 3 && !info_.tagged());
+	bool tainted = info_.tagged() || info_.aux();
+	bool noClause = solver_->isTrue(a) || strategy_ == no_reason || strategy_ == only_reason || (strategy_ == shared_reason && activeClause_.size() > 3 && !tainted);
 	if (noClause) {
 		if (!solver_->force(~a, this))  { return false; }
 		if (strategy_ == only_reason)   { reasons_[a.var()-1].assign(activeClause_.begin()+1, activeClause_.end()); }
@@ -596,15 +597,19 @@ void DefaultUnfoundedCheck::createLoopFormula() {
 		ante = ClauseCreator::create(*solver_, activeClause_, ClauseCreator::clause_no_prepare, info_).local;
 	}
 	else {
-		assert(!info_.tagged());
+		assert(!info_.tagged() && !info_.aux());
 		ante = LoopFormula::newLoopFormula(*solver_
 			, ClauseRep::prepared(&activeClause_[0], (uint32)activeClause_.size(), info_)
 			, &loopAtoms_[0], (uint32)loopAtoms_.size());
 		solver_->addLearnt(ante.constraint(), uint32(loopAtoms_.size() + activeClause_.size()), Constraint_t::Loop);
 	}
 	do {
-		assert(solver_->isTrue(loopAtoms_.back()) && solver_->reason(loopAtoms_.back()) == this);
-		solver_->setReason(loopAtoms_.back(), ante);
+		Literal p = loopAtoms_.back();
+		assert(solver_->isTrue(p));
+		if (solver_->reason(p).asUint() != ante.asUint()) {
+			assert(solver_->reason(p) == this);
+			solver_->setReason(p, ante);
+		}
 		loopAtoms_.pop_back();
 	} while (!loopAtoms_.empty());
 }
