@@ -68,14 +68,12 @@ public:
                 return prg_.showsig(get<Location>(ast, clingo_ast_attribute_location),
                                     Sig(get<String>(ast, clingo_ast_attribute_name),
                                         get<int>(ast, clingo_ast_attribute_arity),
-                                        get<int>(ast, clingo_ast_attribute_positive) == 0),
-                                    ast.hasValue(clingo_ast_attribute_csp) && get<int>(ast, clingo_ast_attribute_csp) != 0);
+                                        get<int>(ast, clingo_ast_attribute_positive) == 0));
             }
             case clingo_ast_type_show_term: {
                 return prg_.show(get<Location>(ast, clingo_ast_attribute_location),
                                  parseTerm(*get<SAST>(ast, clingo_ast_attribute_term)),
-                                 parseBodyLiteralVec(get<AST::ASTVec>(ast, clingo_ast_attribute_body)),
-                                 ast.hasValue(clingo_ast_attribute_csp) && get<int>(ast, clingo_ast_attribute_csp) != 0);
+                                 parseBodyLiteralVec(get<AST::ASTVec>(ast, clingo_ast_attribute_body)));
             }
             case clingo_ast_type_minimize: {
                 return prg_.optimize(get<Location>(ast, clingo_ast_attribute_location),
@@ -140,6 +138,11 @@ public:
                                                                    get<AST::ASTVec>(ast, clingo_ast_attribute_atoms)),
                                       log_);
             }
+            case clingo_ast_type_comment: {
+                return prg_.comment(get<Location>(ast, clingo_ast_attribute_location),
+                             get<String>(ast, clingo_ast_attribute_value),
+                             parseCommentType(get<int>(ast, clingo_ast_attribute_comment_type)) == clingo_comment_type_block);
+            }
             default: {
                 throw std::runtime_error("invalid ast: statement expected");
             }
@@ -278,34 +281,6 @@ private:
         return uid;
     }
 
-    CSPMulTermUid parseCSPMulTerm(AST const &ast) {
-        require_(ast.type() == clingo_ast_type_csp_product, "invalid ast: csp product required");
-        auto const *variable = getOpt(ast, clingo_ast_attribute_variable);
-        return variable != nullptr
-            ? prg_.cspmulterm(get<Location>(ast, clingo_ast_attribute_location),
-                              parseTerm(*get<SAST>(ast, clingo_ast_attribute_coefficient)),
-                              parseTerm(*variable))
-            : prg_.cspmulterm(get<Location>(ast, clingo_ast_attribute_location),
-                              parseTerm(*get<SAST>(ast, clingo_ast_attribute_coefficient)));
-    }
-
-    CSPAddTermUid parseCSPAddTerm(AST const &ast) {
-        require_(ast.type() == clingo_ast_type_csp_sum, "invalid ast: csp sum required");
-        auto const &terms = get<AST::ASTVec>(ast, clingo_ast_attribute_terms);
-        require_(!terms.empty(), "invalid ast: csp sums terms must not be empty");
-        auto it = terms.begin();
-        auto ie = terms.end();
-        auto uid = prg_.cspaddterm(get<Location>(**it, clingo_ast_attribute_location),
-                                   parseCSPMulTerm(**it));
-        for (++it; it != ie; ++it) {
-            uid = prg_.cspaddterm(get<Location>(**it, clingo_ast_attribute_location),
-                                  uid,
-                                  parseCSPMulTerm(**it), true);
-
-        }
-        return uid;
-    }
-
     TheoryTermUid parseTheoryTerm(AST const &ast) {
         switch (ast.type()) {
             case clingo_ast_type_symbolic_term : {
@@ -439,6 +414,24 @@ private:
         }
     }
 
+    RelLitVecUid parseRightGuards(AST::ASTVec const &vec) {
+        if (vec.empty()) {
+            throw std::runtime_error("invalid ast: a comparision must have at least one guard");
+        }
+        auto lhs = get<SAST>(*vec.front(), clingo_ast_attribute_term);
+        auto ret = prg_.rellitvec(get<Location>(*lhs, clingo_ast_attribute_location),
+                                  parseRelation(get<int>(*vec.front(), clingo_ast_attribute_comparison)),
+                                  parseTerm(*lhs));
+        for (auto it = vec.begin() + 1, ie = vec.end(); it != ie; ++it) {
+            auto rhs = get<SAST>(**it, clingo_ast_attribute_term);
+            ret = prg_.rellitvec(get<Location>(*rhs, clingo_ast_attribute_location),
+                                 ret,
+                                 parseRelation(get<int>(**it, clingo_ast_attribute_comparison)),
+                                 parseTerm(*rhs));
+        }
+        return ret;
+    }
+
     LitUid parseLiteral(AST const &ast) {
         switch (ast.type()) {
             case clingo_ast_type_literal: {
@@ -458,34 +451,16 @@ private:
                                             parseAtom(*get<SAST>(ast, clingo_ast_attribute_atom)));
                     }
                     case clingo_ast_type_comparison: {
-                        auto rel = parseRelation(get<int>(atom, clingo_ast_attribute_comparison));
                         return prg_.rellit(loc,
-                                           sign != NAF::NOT ? rel : neg(rel),
-                                           parseTerm(*get<SAST>(atom, clingo_ast_attribute_left)),
-                                           parseTerm(*get<SAST>(atom, clingo_ast_attribute_right)));
+                                           sign,
+                                           parseTerm(*get<SAST>(atom, clingo_ast_attribute_term)),
+                                           parseRightGuards(get<AST::ASTVec>(atom, clingo_ast_attribute_guards)));
                     }
                     default: {
                         throw std::runtime_error("invalid ast: atom expected");
                     }
                 }
             }
-            case clingo_ast_type_csp_literal: {
-                auto const &guards = get<AST::ASTVec>(ast, clingo_ast_attribute_guards);
-                require_(!guards.empty(), "invalid ast: csp literals need at least one guard");
-                auto it = guards.begin();
-                auto ie = guards.end();
-                auto uid = prg_.csplit(get<Location>(ast, clingo_ast_attribute_location),
-                                       parseCSPAddTerm(*get<SAST>(ast, clingo_ast_attribute_term)),
-                                       parseRelation(get<int>(**it, clingo_ast_attribute_comparison)),
-                                       parseCSPAddTerm(*get<SAST>(**it, clingo_ast_attribute_term)));
-                for (++it; it != ie; ++it) {
-                    uid = prg_.csplit(get<Location>(ast, clingo_ast_attribute_location),
-                                      uid,
-                                      parseRelation(get<int>(**it, clingo_ast_attribute_comparison)),
-                                      parseCSPAddTerm(*get<SAST>(**it, clingo_ast_attribute_term)));
-                }
-                return prg_.csplit(uid);
-            }
             default: {
                 throw std::runtime_error("invalid ast: (CSP) literal expected");
             }
@@ -587,19 +562,6 @@ private:
         return uid;
     }
 
-    CSPElemVecUid parseCSPElemVec(AST::ASTVec const &asts) {
-        auto ret = prg_.cspelemvec();
-        for (auto const &ast  : asts) {
-            require_(ast->type() == clingo_ast_type_disjoint_element, "invalid ast: disjoint element expected");
-            ret = prg_.cspelemvec(ret,
-                                  get<Location>(*ast, clingo_ast_attribute_location),
-                                  parseTermVec(get<AST::ASTVec>(*ast, clingo_ast_attribute_terms)),
-                                  parseCSPAddTerm(*get<SAST>(*ast, clingo_ast_attribute_term)),
-                                  parseLiteralVec(get<AST::ASTVec>(*ast, clingo_ast_attribute_condition)));
-        }
-        return ret;
-    }
-
     TheoryAtomUid parseTheoryAtom(AST const &ast) {
         require_(ast.type() == clingo_ast_type_theory_atom, "invalid ast: theory atom expected");
         auto const &loc = get<Location>(ast, clingo_ast_attribute_location);
@@ -620,8 +582,7 @@ private:
 
     HdLitUid parseHeadLiteral(AST const &ast) {
         switch (ast.type()) {
-            case clingo_ast_type_literal:
-            case clingo_ast_type_csp_literal: {
+            case clingo_ast_type_literal: {
                 return prg_.headlit(parseLiteral(ast));
             }
             case clingo_ast_type_disjunction: {
@@ -678,11 +639,6 @@ private:
                             uid = prg_.bodyaggr(uid, loc, sign, parseTheoryAtom(atom));
                             break;
                         }
-                        case clingo_ast_type_disjoint: {
-                            uid = prg_.disjoint(uid, loc, sign,
-                                                parseCSPElemVec(get<AST::ASTVec>(atom, clingo_ast_attribute_elements)));
-                            break;
-                        }
                         default: {
                             uid = prg_.bodylit(uid, parseLiteral(*lit));
                             break;
@@ -797,6 +753,21 @@ private:
         return uid;
     }
 
+    // {{{1 comments
+
+    static clingo_comment_type_t parseCommentType(int num) {
+        switch (num) {
+            case clingo_comment_type_line: {
+                return clingo_comment_type_line;
+            }
+            case clingo_comment_type_block: {
+                return clingo_comment_type_block;
+            }
+            default: {
+                throw std::runtime_error("invalid ast: invalid comment type");
+            }
+        }
+    }
     // 1}}}
 
     Logger &log_;
