@@ -24,39 +24,50 @@
 
 #include "clingo/clingocontrol.hh"
 #include <gringo/input/programbuilder.hh>
+#include "clasp/clingo.h"
 #include "clasp/solver.h"
 #include <potassco/program_opts/typed_value.h>
 #include <potassco/basic_types.h>
 #include <clasp/clause.h>
 #include <clasp/weight_constraint.h>
 #include "clingo.h"
+#include "gringo/backend.hh"
+#include "gringo/hash_set.hh"
+#include "gringo/output/literal.hh"
+#include "gringo/output/literals.hh"
+#include "gringo/output/output.hh"
+#include "gringo/output/statements.hh"
+#include "potassco/theory_data.h"
 #include <csignal>
 #include <clingo/incmode.hh>
+#include <stdexcept>
 
 namespace Gringo {
 
 // {{{1 definition of ClaspAPIBackend
 
-void ClaspAPIBackend::initProgram(bool) { }
+void ClaspAPIBackend::initProgram(bool incremental) {
+    static_cast<void>(incremental);
+}
 
 void ClaspAPIBackend::endStep() { }
 
 void ClaspAPIBackend::beginStep() { }
 
 void ClaspAPIBackend::rule(Potassco::Head_t ht, const Potassco::AtomSpan& head, const Potassco::LitSpan& body) {
-    if (auto p = prg()) { p->addRule(ht, head, body); }
+    if (auto *p = prg()) { p->addRule(ht, head, body); }
 }
 
 void ClaspAPIBackend::rule(Potassco::Head_t ht, const Potassco::AtomSpan& head, Potassco::Weight_t bound, const Potassco::WeightLitSpan& body) {
-    if (auto p = prg()) { p->addRule(ht, head, bound, body); }
+    if (auto *p = prg()) { p->addRule(ht, head, bound, body); }
 }
 
 void ClaspAPIBackend::minimize(Potassco::Weight_t prio, const Potassco::WeightLitSpan& lits) {
-    if (auto p = prg()) { p->addMinimize(prio, lits); }
+    if (auto *p = prg()) { p->addMinimize(prio, lits); }
 }
 
 void ClaspAPIBackend::project(const Potassco::AtomSpan& atoms) {
-    if (auto p = prg()) { p->addProject(atoms); }
+    if (auto *p = prg()) { p->addProject(atoms); }
 }
 
 void ClaspAPIBackend::output(Symbol sym, Potassco::Atom_t atom) {
@@ -64,39 +75,33 @@ void ClaspAPIBackend::output(Symbol sym, Potassco::Atom_t atom) {
     out << sym;
     if (atom != 0) {
         Potassco::Lit_t lit = atom;
-        if (auto p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), Potassco::LitSpan{&lit, 1}); }
+        if (auto *p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), Potassco::LitSpan{&lit, 1}); }
     }
     else {
-        if (auto p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), Potassco::LitSpan{nullptr, 0}); }
+        if (auto *p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), Potassco::LitSpan{nullptr, 0}); }
     }
 }
 
 void ClaspAPIBackend::output(Symbol sym, Potassco::LitSpan const& condition) {
     std::ostringstream out;
     out << sym;
-    if (auto p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), condition); }
-}
-
-void ClaspAPIBackend::output(Symbol sym, int value, Potassco::LitSpan const& condition) {
-    std::ostringstream out;
-    out << sym << "=" << value;
-    if (auto p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), condition); }
+    if (auto *p = prg()) { p->addOutput(Potassco::toSpan(out.str().c_str()), condition); }
 }
 
 void ClaspAPIBackend::acycEdge(int s, int t, const Potassco::LitSpan& condition) {
-    if (auto p = prg()) { p->addAcycEdge(s, t, condition); }
+    if (auto *p = prg()) { p->addAcycEdge(s, t, condition); }
 }
 
 void ClaspAPIBackend::heuristic(Potassco::Atom_t a, Potassco::Heuristic_t t, int bias, unsigned prio, const Potassco::LitSpan& condition) {
-    if (auto p = prg()) { p->addDomHeuristic(a, t, bias, prio, condition); }
+    if (auto *p = prg()) { p->addDomHeuristic(a, t, bias, prio, condition); }
 }
 
 void ClaspAPIBackend::assume(const Potassco::LitSpan& lits) {
-    if (auto p = prg()) { p->addAssumption(lits); }
+    if (auto *p = prg()) { p->addAssumption(lits); }
 }
 
 void ClaspAPIBackend::external(Potassco::Atom_t a, Potassco::Value_t v) {
-    if (auto p = prg()) {
+    if (auto *p = prg()) {
         switch (v) {
             case Potassco::Value_t::False:   { p->freeze(a, Clasp::value_false); break; }
             case Potassco::Value_t::True:    { p->freeze(a, Clasp::value_true); break; }
@@ -131,23 +136,48 @@ ClaspAPIBackend::~ClaspAPIBackend() noexcept = default;
 
 // {{{1 definition of ClingoControl
 
+class ClingoControl::ControlBackend : public Output::ASPIFOutBackend {
+public:
+    ControlBackend(ClingoControl &ctl)
+    : ctl_{ctl} {
+    }
+
+    Output::OutputBase &beginOutput() override {
+        ctl_.beginAddBackend();
+        return *ctl_.out_;
+    }
+
+    void endOutput() override {
+        ctl_.endAddBackend();
+    }
+private:
+    ClingoControl &ctl_;
+};
+
 #define LOG if (verbose_) std::cerr
 ClingoControl::ClingoControl(Scripts &scripts, bool clingoMode, Clasp::ClaspFacade *clasp, Clasp::Cli::ClaspCliConfig &claspConfig, PostGroundFunc pgf, PreSolveFunc psf, Logger::Printer printer, unsigned messageLimit)
 : scripts_(scripts)
 , clasp_(clasp)
 , claspConfig_(claspConfig)
-, pgf_(pgf)
-, psf_(psf)
-, logger_(printer, messageLimit)
+, pgf_(std::move(pgf))
+, psf_(std::move(psf))
+, logger_(std::move(printer), messageLimit)
 , clingoMode_(clingoMode) {
     clasp->ctx.output.theory = &theory_;
 }
 
 void ClingoControl::parse() {
     if (!parser_->empty()) {
-        parser_->parse(logger_);
-        defs_.init(logger_);
-        parsed = true;
+        switch (parser_->parse(logger_)) {
+            case Input::ParseResult::Gringo: {
+                defs_.init(logger_);
+                parsed_ = true;
+                break;
+            }
+            case Input::ParseResult::ASPIF: {
+                break;
+            }
+        }
     }
     if (logger_.hasError()) {
         throw std::runtime_error("parsing failed");
@@ -158,16 +188,15 @@ void ClingoControl::parse(const StringVec& files, const ClingoOptions& opts, Cla
     using namespace Gringo;
     logger_.enable(Warnings::OperationUndefined, !opts.wNoOperationUndefined);
     logger_.enable(Warnings::AtomUndefined, !opts.wNoAtomUndef);
-    logger_.enable(Warnings::VariableUnbounded, !opts.wNoVariableUnbounded);
     logger_.enable(Warnings::FileIncluded, !opts.wNoFileIncluded);
     logger_.enable(Warnings::GlobalVariable, !opts.wNoGlobalVariable);
     logger_.enable(Warnings::Other, !opts.wNoOther);
     verbose_ = opts.verbose;
     Output::OutputPredicates outPreds;
-    for (auto &x : opts.foobar) {
-        outPreds.emplace_back(Location("<cmd>",1,1,"<cmd>", 1,1), x, false);
+    for (auto const &x : opts.sigvec) {
+        outPreds.add(Location("<cmd>",1,1,"<cmd>", 1,1), x, false);
     }
-    if (claspOut) {
+    if (claspOut != nullptr) {
         out_ = gringo_make_unique<Output::OutputBase>(claspOut->theoryData(), std::move(outPreds), gringo_make_unique<ClaspAPIBackend>(*this), opts.outputOptions);
     }
     else {
@@ -175,9 +204,10 @@ void ClingoControl::parse(const StringVec& files, const ClingoOptions& opts, Cla
         out_ = gringo_make_unique<Output::OutputBase>(*data_, std::move(outPreds), std::cout, opts.outputFormat, opts.outputOptions);
     }
     out_->keepFacts = opts.keepFacts;
-    pb_ = gringo_make_unique<Input::NongroundProgramBuilder>(scripts_, prg_, *out_, defs_, opts.rewriteMinimize);
-    parser_ = gringo_make_unique<Input::NonGroundParser>(*pb_, incmode_);
-    for (auto &x : opts.defines) {
+    aspif_bck_ = gringo_make_unique<ControlBackend>(*this);
+    pb_ = gringo_make_unique<Input::NongroundProgramBuilder>(scripts_, prg_, out_->outPreds, defs_, opts.rewriteMinimize);
+    parser_ = gringo_make_unique<Input::NonGroundParser>(*pb_, *aspif_bck_, incmode_);
+    for (auto const &x : opts.defines) {
         LOG << "define: " << x << std::endl;
         parser_->parseDefine(x, logger_);
     }
@@ -200,20 +230,20 @@ bool ClingoControl::update() {
         configUpdate_ = false;
         if (!clasp_->ok()) { return false; }
     }
-    if (!grounded) {
+    if (!grounded_) {
         if (!initialized_) {
             out_->init(clasp_->incremental());
             initialized_ = true;
         }
         out_->beginStep();
-        grounded = true;
+        grounded_ = true;
     }
     return true;
 }
 
 void ClingoControl::ground(Control::GroundVec const &parts, Context *context) {
     if (!update()) { return; }
-    if (parsed) {
+    if (parsed_) {
         LOG << "************** parsed program **************" << std::endl << prg_;
         prg_.rewrite(defs_, logger_);
         LOG << "************* rewritten program ************" << std::endl << prg_;
@@ -221,22 +251,21 @@ void ClingoControl::ground(Control::GroundVec const &parts, Context *context) {
         if (logger_.hasError()) {
             throw std::runtime_error("grounding stopped because of errors");
         }
-        parsed = false;
+        parsed_ = false;
     }
     if (!parts.empty()) {
         Ground::Parameters params;
         std::set<Sig> sigs;
-        for (auto &x : parts) {
+        for (auto const &x : parts) {
             params.add(x.first, SymVec(x.second));
             sigs.emplace(x.first, numeric_cast<uint32_t>(x.second.size()), false);
         }
         auto gPrg = prg_.toGround(sigs, out_->data, logger_);
-        LOG << "*********** intermediate program ***********" << std::endl << gPrg << std::endl;
+        LOG << "*********** intermediate program ***********" << std::endl
+            << gPrg << std::endl;
         LOG << "************* grounded program *************" << std::endl;
-        auto exit = onExit([this]{ scripts_.resetContext(); });
-        if (context) { scripts_.setContext(*context); }
         gPrg.prepare(params, *out_, logger_);
-        gPrg.ground(scripts_, *out_, logger_);
+        scripts_.withContext(context, [&, this](Context &ctx) { gPrg.ground(ctx, *out_, logger_); });
     }
 }
 
@@ -458,11 +487,15 @@ public:
     void setCheckMode(clingo_propagator_check_mode_t checkMode) override {
         p_.enableClingoPropagatorCheck(static_cast<Clasp::ClingoPropagatorCheck_t::Type>(checkMode));
     }
+    void setUndoMode(clingo_propagator_undo_mode_t undoMode) override {
+        p_.enableClingoPropagatorUndo(static_cast<Clasp::ClingoPropagatorUndo_t::Type>(undoMode));
+    }
     Potassco::AbstractAssignment const &assignment() const override { return a_; }
     clingo_propagator_check_mode_t getCheckMode() const override { return p_.checkMode(); }
+    clingo_propagator_undo_mode_t getUndoMode() const override { return p_.undoMode(); }
 private:
     Clasp::ClaspFacade &facade_() const { return *static_cast<ClingoControl&>(c_).clasp_; }
-private:
+
     Control &c_;
     Clasp::ClingoPropagatorInit &p_;
     Clasp::ClingoAssignment a_;
@@ -475,7 +508,7 @@ void ClingoControl::prepare(Assumptions ass) {
     eventHandler_ = nullptr;
     // finalize the program
     if (update()) { out_->endStep(ass); }
-    grounded = false;
+    grounded_ = false;
     if (clingoMode_) {
         Clasp::ProgramBuilder *prg = clasp_->program();
         postGround(*prg);
@@ -564,7 +597,8 @@ std::string ClingoControl::str() {
 
 void ClingoControl::assignExternal(Potassco::Atom_t ext, Potassco::Value_t val) {
     if (update()) {
-        if (auto backend = out_->backend()) {
+        auto *backend = out_->backend();
+        if (backend != nullptr) {
             backend->external(ext, val);
         }
     }
@@ -659,7 +693,7 @@ void advance(Output::OutputBase &out, SymbolicAtomIter &it) {
 
 Output::PredicateAtom &domainElem(Output::PredDomMap &map, SymbolicAtomIter it) {
     auto &off = toOffset(it).data;
-    return (*map[off.domain_offset])[off.atom_offset];
+    return map.nth(off.domain_offset).key()->operator[](off.atom_offset);
 }
 
 } // namespace
@@ -689,7 +723,7 @@ SymbolicAtomIter ClingoControl::next(SymbolicAtomIter it) const {
 
 bool ClingoControl::valid(SymbolicAtomIter it) const {
     auto &off = toOffset(it).data;
-    return off.domain_offset < out_->predDoms().size() && off.atom_offset < out_->predDoms()[off.domain_offset]->size();
+    return off.domain_offset < out_->predDoms().size() && off.atom_offset < out_->predDom(off.domain_offset).size();
 }
 
 std::vector<Sig> ClingoControl::signatures() const {
@@ -701,7 +735,12 @@ std::vector<Sig> ClingoControl::signatures() const {
 }
 
 SymbolicAtomIter ClingoControl::begin(Sig sig) const {
-    return init(*out_, out_->predDoms().offset(out_->predDoms().find(sig)), false);
+    auto &doms = out_->predDoms();
+    auto it = doms.find(sig);
+    if (it != doms.end()) {
+        return init(*out_, it.key()->domainOffset(), false);
+    }
+    return init(*out_, numeric_cast<uint32_t>(doms.size()), false);
 }
 
 SymbolicAtomIter ClingoControl::begin() const {
@@ -709,7 +748,7 @@ SymbolicAtomIter ClingoControl::begin() const {
 }
 
 SymbolicAtomIter ClingoControl::end() const {
-    return SymbolicAtomOffset{out_->predDoms().size(), false, 0, false}.repr;
+    return SymbolicAtomOffset{numeric_cast<uint32_t>(out_->predDoms().size()), false, 0, false}.repr;
 }
 
 bool ClingoControl::eq(SymbolicAtomIter it, SymbolicAtomIter jt) const {
@@ -722,7 +761,7 @@ SymbolicAtomIter ClingoControl::lookup(Symbol atom) const {
         if (it != out_->predDoms().end()) {
             auto jt = (*it)->find(atom);
             if (jt != (*it)->end()) {
-                return SymbolicAtomOffset(out_->predDoms().offset(it), true, numeric_cast<uint32_t>(jt - (*it)->begin()), true).repr;
+                return SymbolicAtomOffset(it.key()->domainOffset(), true, numeric_cast<uint32_t>(jt - (*it)->begin()), true).repr;
             }
         }
     }
@@ -874,27 +913,36 @@ void ClingoLib::initOptions(Potassco::ProgramOptions::OptionContext& root) {
     gringo.addOptions()
         ("verbose,V"                , flag(grOpts_.verbose = false), "Enable verbose output")
         ("const,c"                  , storeTo(grOpts_.defines, parseConst)->composing()->arg("<id>=<term>"), "Replace term occurrences of <id> with <term>")
-        ("output-debug", storeTo(grOpts_.outputOptions.debug = Output::OutputDebug::NONE, values<Output::OutputDebug>()
+        ("output-debug"             , storeTo(grOpts_.outputOptions.debug = Output::OutputDebug::NONE, values<Output::OutputDebug>()
           ("none", Output::OutputDebug::NONE)
           ("text", Output::OutputDebug::TEXT)
           ("translate", Output::OutputDebug::TRANSLATE)
-          ("all", Output::OutputDebug::ALL)), "Print debug information during output:\n"
+          ("all", Output::OutputDebug::ALL)),
+         "Print debug information during output:\n"
          "      none     : no additional info\n"
          "      text     : print rules as plain text (prefix %%)\n"
          "      translate: print translated rules as plain text (prefix %%%%)\n"
          "      all      : combines text and translate")
-        ("warn,W"                   , storeTo(grOpts_, parseWarning)->arg("<warn>")->composing(), "Enable/disable warnings:\n"
-         "      none:                     disable all warnings\n"
-         "      all:                      enable all warnings\n"
-         "      [no-]atom-undefined:      a :- b.\n"
-         "      [no-]file-included:       #include \"a.lp\". #include \"a.lp\".\n"
+        ("warn,W"                   , storeTo(grOpts_, parseWarning)->arg("<warn>")->composing(),
+         "Enable/disable warnings:\n"
+         "      none                    : disable all warnings\n"
+         "      all                     : enable all warnings\n"
+         "      [no-]atom-undefined     : a :- b.\n"
+         "      [no-]file-included      : #include \"a.lp\". #include \"a.lp\".\n"
          "      [no-]operation-undefined: p(1/0).\n"
-         "      [no-]variable-unbounded:  $x > 10.\n"
-         "      [no-]global-variable:     :- #count { X } = 1, X = 1.\n"
-         "      [no-]other:               clasp related and uncategorized warnings")
+         "      [no-]global-variable    : :- #count { X } = 1, X = 1.\n"
+         "      [no-]other              : clasp related and uncategorized warnings")
         ("rewrite-minimize"         , flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
-        ("keep-facts"               , flag(grOpts_.keepFacts = false), "Do not remove facts from normal rules")
-        ("single-shot,@2"           , flag(grOpts_.singleShot = false), "Force single-shot solving mode")
+        // for backward compatibility
+        ("keep-facts"               , flag(grOpts_.keepFacts = false), "Preserve facts in rule bodies")
+        ("preserve-facts"           , storeTo(grOpts_, parsePreserveFacts),
+         "Preserve facts in output:\n"
+         "      none  : do not preserve\n"
+         "      body  : do not preserve\n"
+         "      symtab: do not preserve\n"
+         "      all   : preserve all facts")
+        ("single-shot"              , flag(grOpts_.singleShot = false), "Force single-shot solving mode")
+        ("show-preds"               , storeTo(grOpts_.sigvec, parseSigVec), "Show the given signatures")
         ;
     root.add(gringo);
     claspConfig_.addOptions(root);
