@@ -22,6 +22,7 @@
 
 // }}}
 
+#include "potassco/basic_types.h"
 #if defined(__GNUC__) && !defined(__clang__)
 #pragma GCC diagnostic push
 #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
@@ -42,13 +43,21 @@
 #   include <mutex>
 #endif
 
+#define CLINGO_QUOTE_(name) #name
+#define CLINGO_QUOTE(name) CLINGO_QUOTE_(name)
+#ifdef CLINGO_BUILD_REVISION
+#   define CLINGO_VERSION_STRING CLINGO_VERSION " (" CLINGO_QUOTE(CLINGO_BUILD_REVISION) ")"
+#else
+#   define CLINGO_VERSION_STRING CLINGO_VERSION
+#endif
+
 // {{{1 error handling
 
 using namespace Gringo;
 
 namespace {
 
-// {{{1 declaration of ClingoError
+// {{{2 declaration of ClingoError
 
 struct ClingoError : std::exception {
     ClingoError()
@@ -152,8 +161,7 @@ void clingo_terminate(char const *loc) {
     std::_Exit(1);
 }
 
-// }}}1
-// {{{1
+// {{{2
 
 clingo_location_t conv(Location const &loc) {
     return {loc.beginFilename.c_str(), loc.endFilename.c_str(), loc.beginLine, loc.endLine, loc.beginColumn, loc.endColumn};
@@ -168,9 +176,20 @@ Location conv(clingo_location_t const &loc) {
              static_cast<unsigned int>(loc.end_column)};
 }
 
-// 1}}}
 } // namespace
 
+// 2}}}
+
+// 1}}}
+
+namespace Gringo {
+
+char const *clingo_version_string() noexcept {
+    return CLINGO_VERSION_STRING;
+}
+
+} // namespace Gringo
+
 // c interface
 
 // {{{1 error handling
@@ -692,10 +711,18 @@ extern "C" void clingo_propagate_init_set_check_mode(clingo_propagate_init_t *in
     init->setCheckMode(mode);
 }
 
+extern "C" void clingo_propagate_init_set_undo_mode(clingo_propagate_init_t *init, clingo_propagator_undo_mode_t mode) {
+    init->setUndoMode(mode);
+}
+
 extern "C" clingo_propagator_check_mode_t clingo_propagate_init_get_check_mode(clingo_propagate_init_t const *init) {
     return init->getCheckMode();
 }
 
+extern "C" clingo_propagator_undo_mode_t clingo_propagate_init_get_undo_mode(clingo_propagate_init_t const *init) {
+    return init->getUndoMode();
+}
+
 extern "C" clingo_assignment_t const *clingo_propagate_init_assignment(clingo_propagate_init_t const *init) {
     return static_cast<clingo_assignment_t const *>(&init->assignment());
 }
@@ -831,6 +858,15 @@ extern "C" bool clingo_model_cost(clingo_model_t const *m, int64_t *ret, size_t
     GRINGO_CLINGO_CATCH;
 }
 
+extern "C" bool clingo_model_priority(clingo_model_t const *model, clingo_weight_t *priorities, size_t size) {
+    GRINGO_CLINGO_TRY {
+        auto vec = model->priorities();
+        if (size < vec.size()) { throw std::length_error("not enough space"); }
+        std::copy(vec.begin(), vec.end(), priorities);
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
 extern "C" bool clingo_model_extend(clingo_model_t *model, clingo_symbol_t const *symbols, size_t size) {
     GRINGO_CLINGO_TRY { model->add({reinterpret_cast<Symbol const *>(symbols), size}); }
     GRINGO_CLINGO_CATCH;
@@ -852,7 +888,11 @@ extern "C" bool clingo_model_type(clingo_model_t const *m, clingo_model_type_t *
 }
 
 extern "C" bool clingo_model_is_true(clingo_model_t const *m, clingo_literal_t lit, bool *result) {
-    GRINGO_CLINGO_TRY { *result = static_cast<clingo_model_type_t>(m->isTrue(lit)); }
+    GRINGO_CLINGO_TRY { *result = m->isTrue(lit); }
+    GRINGO_CLINGO_CATCH;
+}
+extern "C" bool clingo_model_is_consequence(clingo_model_t const *model, clingo_literal_t literal, clingo_consequence_t *result) {
+    GRINGO_CLINGO_TRY { *result = static_cast<clingo_consequence_t>(model->isConsequence(literal)); }
     GRINGO_CLINGO_CATCH;
 }
 
@@ -1111,6 +1151,68 @@ extern "C" bool clingo_backend_acyc_edge(clingo_backend_t *backend, int node_u,
     GRINGO_CLINGO_CATCH;
 }
 
+extern "C" bool clingo_backend_theory_term_number(clingo_backend_t *backend, int number, clingo_id_t *term_id) {
+    GRINGO_CLINGO_TRY { *term_id = backend->theoryData().addTerm(number); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_term_string(clingo_backend_t *backend, char const *string, clingo_id_t *term_id) {
+    GRINGO_CLINGO_TRY { *term_id = backend->theoryData().addTerm(string); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_term_function(clingo_backend_t *backend, char const *name, clingo_id_t const *arguments, size_t size, clingo_id_t *term_id) {
+    GRINGO_CLINGO_TRY {
+        auto &theory = backend->theoryData();
+        auto name_id  = theory.addTerm(name);
+        *term_id = theory.addTermFun(name_id, Potassco::IdSpan{arguments, size});
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_term_symbol(clingo_backend_t *backend, clingo_symbol_t symbol, clingo_id_t *term_id) {
+    GRINGO_CLINGO_TRY {
+        *term_id  = backend->theoryData().addTerm(Gringo::Symbol{symbol});
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_term_sequence(clingo_backend_t *backend, clingo_theory_sequence_type_t type, clingo_id_t const *arguments, size_t size, clingo_id_t *term_id) {
+    GRINGO_CLINGO_TRY {
+        *term_id  = backend->theoryData().addTermTup(Potassco::Tuple_t{-static_cast<int>(type) - 1}, Potassco::IdSpan{arguments, size});
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_element(clingo_backend_t *backend, clingo_id_t const *tuple, size_t tuple_size, clingo_literal_t const *condition, size_t condition_size, clingo_id_t *element_id) {
+    GRINGO_CLINGO_TRY {
+        *element_id = backend->theoryData().addElem(Potassco::IdSpan{tuple, tuple_size}, Potassco::LitSpan{condition, condition_size});
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_atom(clingo_backend_t *backend, clingo_atom_t atom, clingo_id_t term_id, clingo_id_t const *elements, size_t size, clingo_atom_t *atom_id) {
+    GRINGO_CLINGO_TRY {
+        auto newAtom = [backend, atom]() -> Atom_t {
+            return atom == std::numeric_limits<clingo_atom_t>::max() ? backend->addProgramAtom() : atom;
+        };
+        *atom_id = backend->theoryData().addAtom(newAtom, term_id, Potassco::IdSpan{elements, size}).first.atom();
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_theory_atom_with_guard(clingo_backend_t *backend, clingo_atom_t atom, clingo_id_t term_id, clingo_id_t const *elements, size_t size, char const *operator_name, clingo_id_t right_hand_side_id, clingo_atom_t *atom_id) {
+    GRINGO_CLINGO_TRY {
+        auto &theory = backend->theoryData();
+        auto op_id = theory.addTerm(operator_name);
+        auto newAtom = [backend, atom]() -> Atom_t {
+            return atom == std::numeric_limits<clingo_atom_t>::max() ? backend->addProgramAtom() : atom;
+        };
+        *atom_id = theory.addAtom(newAtom, term_id, Potassco::IdSpan{elements, size}, op_id, right_hand_side_id).first.atom();
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
 extern "C" bool clingo_backend_add_atom(clingo_backend_t *backend, clingo_symbol_t *symbol, clingo_atom_t *ret) {
     GRINGO_CLINGO_TRY {
         if (symbol) {
@@ -1181,17 +1283,12 @@ C(binary_operation) { A(location, location), A(operator_type, number), A(left, a
 C(interval) { A(location, location), A(left, ast), A(right, ast) };
 C(function) { A(location, location), A(name, string), A(arguments, ast_array), A(external, number) };
 C(pool) { A(location, location), A(arguments, ast_array) };
-// csp terms
-C(csp_product) { A(location, location), A(coefficient, ast), A(variable, optional_ast) };
-C(csp_sum) { A(location, location), A(terms, ast_array) };
-C(csp_guard) { A(comparison, number), A(term, ast) };
 // simple atoms
 C(boolean_constant) { A(value, number) };
 C(symbolic_atom) { A(symbol, ast) };
-C(comparison) { A(comparison, number), A(left, ast), A(right, ast) };
-C(csp_literal) { A(location, location), A(term, ast), A(guards, ast_array) };
+C(comparison) { A(term, ast), A(guards, ast_array) };
 // aggregates
-C(aggregate_guard) { A(comparison, number), A(term, ast) };
+C(guard) { A(comparison, number), A(term, ast) };
 C(conditional_literal) { A(location, location), A(literal, ast), A(condition, ast_array) };
 C(aggregate) { A(location, location), A(left_guard, optional_ast), A(elements, ast_array), A(right_guard, optional_ast) };
 C(body_aggregate_element) { A(terms, ast_array), A(condition, ast_array) };
@@ -1199,8 +1296,6 @@ C(body_aggregate) { A(location, location), A(left_guard, optional_ast), A(functi
 C(head_aggregate_element) { A(terms, ast_array), A(condition, ast) };
 C(head_aggregate) { A(location, location), A(left_guard, optional_ast), A(function, number), A(elements, ast_array), A(right_guard, optional_ast) };
 C(disjunction) { A(location, location), A(elements, ast_array) };
-C(disjoint_element) { A(location, location), A(terms, ast_array), A(term, ast), A(condition, ast_array) };
-C(disjoint) { A(location, location), A(elements, ast_array) };
 // theory atoms
 C(theory_sequence) { A(location, location), A(sequence_type, number), A(terms, ast_array) };
 C(theory_function) { A(location, location), A(name, string), A(arguments, ast_array) };
@@ -1219,8 +1314,8 @@ C(theory_atom_definition) { A(location, location), A(atom_type, number), A(name,
 // statemets
 C(rule) { A(location, location), A(head, ast), A(body, ast_array) };
 C(definition) { A(location, location), A(name, string), A(value, ast), A(is_default, number) };
-C(show_signature) { A(location, location), A(name, string), A(arity, number), A(positive, number), A(csp, number) };
-C(show_term) { A(location, location), A(term, ast), A(body, ast_array), A(csp, number) };
+C(show_signature) { A(location, location), A(name, string), A(arity, number), A(positive, number) };
+C(show_term) { A(location, location), A(term, ast), A(body, ast_array) };
 C(minimize) { A(location, location), A(weight, ast), A(priority, ast), A(terms, ast_array), A(body, ast_array) };
 C(script) { A(location, location), A(name, string), A(code, string) };
 C(program) { A(location, location), A(name, string), A(parameters, ast_array) };
@@ -1231,6 +1326,7 @@ C(project_atom) { A(location, location), A(atom, ast), A(body, ast_array) };
 C(project_signature) { A(location, location), A(name, string), A(arity, number), A(positive, number) };
 C(defined) { A(location, location), A(name, string), A(arity, number), A(positive, number) };
 C(theory_definition) { A(location, location), A(name, string), A(terms, ast_array), A(atoms, ast_array) };
+C(comment) { A(location, location), A(value, string), A(comment_type, number) };
 
 clingo_ast_constructor_t const clingo_ast_constructor_list[] = {
     // terms
@@ -1242,17 +1338,12 @@ clingo_ast_constructor_t const clingo_ast_constructor_list[] = {
     E(interval),
     E(function),
     E(pool),
-    // csp terms
-    E(csp_product),
-    E(csp_sum),
-    E(csp_guard),
     // simple atoms
     E(boolean_constant),
     E(symbolic_atom),
     E(comparison),
-    E(csp_literal),
     // aggregates
-    E(aggregate_guard),
+    E(guard),
     E(conditional_literal),
     E(aggregate),
     E(body_aggregate_element),
@@ -1260,8 +1351,6 @@ clingo_ast_constructor_t const clingo_ast_constructor_list[] = {
     E(head_aggregate_element),
     E(head_aggregate),
     E(disjunction),
-    E(disjoint_element),
-    E(disjoint),
     // theory atoms
     E(theory_sequence),
     E(theory_function),
@@ -1291,7 +1380,8 @@ clingo_ast_constructor_t const clingo_ast_constructor_list[] = {
     E(project_atom),
     E(project_signature),
     E(defined),
-    E(theory_definition)
+    E(theory_definition),
+    E(comment)
 };
 
 clingo_ast_constructors_t g_clingo_ast_constructors = {
@@ -1311,7 +1401,6 @@ char const * attribute_list[] = {
     "coefficient",
     "comparison",
     "condition",
-    "csp",
     "elements",
     "external",
     "external_type",
@@ -1344,6 +1433,7 @@ char const * attribute_list[] = {
     "value",
     "variable",
     "weight",
+    "comment_type",
 };
 
 clingo_ast_attribute_names_t g_clingo_ast_attribute_names = {
@@ -1653,13 +1743,27 @@ extern "C" bool clingo_ast_attribute_insert_ast_at(clingo_ast_t *ast, clingo_ast
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_ast_parse_string(char const *program, clingo_ast_callback_t cb, void *cb_data, clingo_logger_t logger, void *logger_data, unsigned message_limit) {
+namespace {
+
+Backend &get_backend(clingo_control_t *control) {
+    static NullBackend null_bck;
+    Backend *bck = &null_bck;
+    if (control != nullptr) {
+        bck = &control->getASPIFBackend();
+    }
+    return *bck;
+}
+
+} // namespace
+
+extern "C" bool clingo_ast_parse_string(char const *program, clingo_ast_callback_t cb, void *cb_data, clingo_control_t *control, clingo_logger_t logger, void *logger_data, unsigned message_limit) {
     GRINGO_CLINGO_TRY {
         auto builder = Input::build([cb, cb_data](Input::SAST ast) {
             forwardError(cb(reinterpret_cast<clingo_ast_t*>(ast.get()), cb_data));
         });
         bool incmode = false;
-        Input::NonGroundParser parser{*builder, incmode};
+        Input::NonGroundParser parser{*builder, get_backend(control), incmode};
+        parser.disable_aspif();
         Logger::Printer printer;
         if (logger != nullptr) { printer = [logger, logger_data](Warnings code, char const *msg) { logger(static_cast<clingo_warning_t>(code), msg, logger_data); }; }
         Logger log(printer, message_limit);
@@ -1670,13 +1774,14 @@ extern "C" bool clingo_ast_parse_string(char const *program, clingo_ast_callback
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_ast_parse_files(char const * const *file, size_t n, clingo_ast_callback_t cb, void *cb_data, clingo_logger_t logger, void *logger_data, unsigned message_limit) {
+extern "C" bool clingo_ast_parse_files(char const * const *file, size_t n, clingo_ast_callback_t cb, void *cb_data, clingo_control_t *control, clingo_logger_t logger, void *logger_data, unsigned message_limit) {
     GRINGO_CLINGO_TRY {
         auto builder = Input::build([cb, cb_data](Input::SAST ast) {
             forwardError(cb(reinterpret_cast<clingo_ast_t*>(ast.get()), cb_data));
         });
         bool incmode = false;
-        Input::NonGroundParser parser(*builder, incmode);
+        Input::NonGroundParser parser(*builder, get_backend(control), incmode);
+        parser.disable_aspif();
         Logger::Printer printer;
         if (logger != nullptr) { printer = [logger, logger_data](Warnings code, char const *msg) { logger(static_cast<clingo_warning_t>(code), msg, logger_data); }; }
         Logger log(printer, message_limit);
@@ -1860,7 +1965,7 @@ extern "C" bool clingo_control_release_external(clingo_control_t *ctl, clingo_li
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_control_program_builder(clingo_control_t *ctl, clingo_program_builder_t **ret) {
+extern "C" bool clingo_program_builder_init(clingo_control_t *ctl, clingo_program_builder_t **ret) {
     GRINGO_CLINGO_TRY { *ret = static_cast<clingo_program_builder_t*>(ctl); }
     GRINGO_CLINGO_CATCH;
 }
@@ -2026,9 +2131,6 @@ public:
     void output(Symbol sym, Potassco::LitSpan const& condition) override {
         call(obs_.output_term, sym.rep(), condition.first, condition.size);
     }
-    void output(Symbol sym, int value, Potassco::LitSpan const& condition) override {
-        call(obs_.output_csp, sym.rep(), value, condition.first, condition.size);
-    }
     void external(Atom_t a, Potassco::Value_t v) override {
         call(obs_.external, a, v);
     }
