@@ -22,6 +22,7 @@
 
 // }}}
 
+#include "clingo.h"
 #include <clingo/incmode.hh>
 #include <clingo/control.hh>
 #include <clingo/astv2.hh>
@@ -45,7 +46,7 @@ namespace Gringo {
 using StrVec = std::vector<std::string>;
 
 struct GringoOptions {
-    using Foobar = std::vector<Sig>;
+    using SigVec = std::vector<Sig>;
     StrVec                     defines;
     Output::OutputOptions outputOptions;
     Output::OutputFormat  outputFormat          = Output::OutputFormat::INTERMEDIATE;
@@ -53,13 +54,12 @@ struct GringoOptions {
     bool                          wNoOperationUndefined = false;
     bool                          wNoAtomUndef          = false;
     bool                          wNoFileIncluded       = false;
-    bool                          wNoVariableUnbounded  = false;
     bool                          wNoGlobalVariable     = false;
     bool                          wNoOther              = false;
     bool                          rewriteMinimize       = false;
     bool                          keepFacts             = false;
     bool                          singleShot            = false;
-    Foobar                        foobar;
+    SigVec                        sigvec;
 };
 
 static inline std::vector<std::string> split(std::string const &source, char const *delimiter = " ", bool keepEmpty = false) {
@@ -74,7 +74,7 @@ static inline std::vector<std::string> split(std::string const &source, char con
     return results;
 }
 
-static inline bool parseFoobar(const std::string& str, GringoOptions::Foobar& foobar) {
+static inline bool parseSigVec(const std::string& str, GringoOptions::SigVec& sigvec) {
     for (auto &x : split(str, ",")) {
         auto y = split(x, "/");
         if (y.size() != 2) { return false; }
@@ -82,25 +82,24 @@ static inline bool parseFoobar(const std::string& str, GringoOptions::Foobar& fo
         if (!Potassco::string_cast<unsigned>(y[1], a)) { return false; }
         bool sign = !y[0].empty() && y[0][0] == '-';
         if (sign) { y[0] = y[0].substr(1); }
-        foobar.emplace_back(y[0].c_str(), a, sign);
+        sigvec.emplace_back(y[0].c_str(), a, sign);
     }
     return true;
 }
 
 #define LOG if (opts.verbose) std::cerr
-struct IncrementalControl : Control {
+struct IncrementalControl : Control, private Output::ASPIFOutBackend {
     IncrementalControl(Output::OutputBase &out, StrVec const &files, GringoOptions const &opts)
     : out(out)
     , scripts(g_scripts())
-    , pb(scripts, prg, out, defs, opts.rewriteMinimize)
-    , parser(pb, incmode)
+    , pb(scripts, prg, out.outPreds, defs, opts.rewriteMinimize)
+    , parser(pb, *this, incmode)
     , opts(opts) {
         using namespace Gringo;
         // TODO: should go where python script is once refactored
         out.keepFacts = opts.keepFacts;
         logger_.enable(Warnings::OperationUndefined, !opts.wNoOperationUndefined);
         logger_.enable(Warnings::AtomUndefined, !opts.wNoAtomUndef);
-        logger_.enable(Warnings::VariableUnbounded, !opts.wNoVariableUnbounded);
         logger_.enable(Warnings::FileIncluded, !opts.wNoFileIncluded);
         logger_.enable(Warnings::GlobalVariable, !opts.wNoGlobalVariable);
         logger_.enable(Warnings::Other, !opts.wNoOther);
@@ -118,21 +117,53 @@ struct IncrementalControl : Control {
         }
         parse();
     }
+    Backend &getASPIFBackend() override {
+        return *this;
+    }
+    Output::OutputBase &beginOutput() override {
+        beginAddBackend();
+        return out;
+    }
+    void endOutput() override {
+        endAddBackend();
+    }
+
     Logger &logger() override {
         return logger_;
     }
+    void update() {
+        // This function starts a new step and has to be called at least once
+        // before anything that causes output at the beginning of excecution or
+        // after a solve step.
+        if (!grounded) {
+            if (!initialized_) {
+                initialized_ = true;
+                out.init(incremental_);
+            }
+            out.beginStep();
+            grounded = true;
+        }
+    }
     void parse() {
         if (!parser.empty()) {
-            parser.parse(logger_);
-            defs.init(logger_);
-            parsed = true;
+            switch (parser.parse(logger_)) {
+                case Input::ParseResult::Gringo: {
+                    defs.init(logger_);
+                    parsed = true;
+                    break;
+                }
+                case Input::ParseResult::ASPIF: {
+                    break;
+                }
+            }
+        }
+        if (logger_.hasError()) {
+            throw std::runtime_error("parsing failed");
         }
     }
     void ground(Control::GroundVec const &parts, Context *context) override {
+        update();
         // NOTE: it would be cool to have assumptions in the lparse output
-        auto exit = onExit([this]{ scripts.resetContext(); });
-        if (context) { scripts.setContext(*context); }
-        parse();
         if (parsed) {
             LOG << "************** parsed program **************" << std::endl << prg;
             prg.rewrite(defs, logger_);
@@ -143,14 +174,6 @@ struct IncrementalControl : Control {
             }
             parsed = false;
         }
-        if (!grounded) {
-            if (!initialized_) {
-                initialized_ = true;
-                out.init(incremental_);
-            }
-            out.beginStep();
-            grounded = true;
-        }
         if (!parts.empty()) {
             Ground::Parameters params;
             std::set<Sig> sigs;
@@ -162,7 +185,7 @@ struct IncrementalControl : Control {
             LOG << "************* intermediate program *************" << std::endl << gPrg << std::endl;
             LOG << "*************** grounded program ***************" << std::endl;
             gPrg.prepare(params, out, logger_);
-            gPrg.ground(scripts, out, logger_);
+            scripts.withContext(context, [&, this](Context &ctx) { gPrg.ground(ctx, out, logger_); });
         }
     }
     void add(std::string const &name, StringVec const &params, std::string const &part) override {
@@ -187,6 +210,7 @@ struct IncrementalControl : Control {
     }
     bool blocked() override { return false; }
     USolveFuture solve(Assumptions ass, clingo_solve_mode_bitset_t, USolveEventHandler cb) override {
+        update();
         grounded = false;
         out.endStep(ass);
         out.reset(true);
@@ -211,6 +235,7 @@ struct IncrementalControl : Control {
     Potassco::AbstractStatistics const *statistics() const override { throw std::runtime_error("statistics not supported (yet)"); }
     bool isConflicting() const noexcept override { return false; }
     void assignExternal(Potassco::Atom_t ext, Potassco::Value_t val) override {
+        update();
         if (auto *b = out.backend()) { b->external(ext, val); }
     }
     SymbolicAtoms const &getDomain() const override { throw std::runtime_error("domain introspection not supported"); }
@@ -224,6 +249,7 @@ struct IncrementalControl : Control {
     virtual ~IncrementalControl() { }
     Output::DomainData const &theory() const override { return out.data; }
     bool beginAddBackend() override {
+        update();
         backend_prg_ = std::make_unique<Ground::Program>(prg.toGround({}, out.data, logger_));
         backend_prg_->prepare({}, out, logger_);
         backend_ = out.backend();
@@ -233,6 +259,9 @@ struct IncrementalControl : Control {
         if (!backend_) { throw std::runtime_error("backend not available"); }
         return backend_;
     }
+    Output::TheoryData &theoryData() override {
+        return out.data.theory();
+    }
     Id_t addAtom(Symbol sym) override {
         bool added = false;
         auto atom  = out.addAtom(sym, &added);
@@ -275,11 +304,11 @@ struct IncrementalControl : Control {
     Backend                       *backend_ = nullptr;
     std::unique_ptr<Ground::Program> backend_prg_;
     std::unique_ptr<Input::NongroundProgramBuilder> builder;
-    bool                                   incmode = false;
-    bool                                   parsed = false;
-    bool                                   grounded = false;
+    bool incmode = false;
+    bool parsed = false;
+    bool grounded = false;
     bool initialized_ = false;
-    bool incremental_ = true;
+    bool incremental_ = false;
 };
 #undef LOG
 
@@ -292,7 +321,6 @@ inline void enableAll(GringoOptions& out, bool enable) {
     out.wNoAtomUndef          = !enable;
     out.wNoFileIncluded       = !enable;
     out.wNoOperationUndefined = !enable;
-    out.wNoVariableUnbounded  = !enable;
     out.wNoGlobalVariable     = !enable;
     out.wNoOther              = !enable;
 }
@@ -306,8 +334,6 @@ inline bool parseWarning(const std::string& str, GringoOptions& out) {
     if (str ==    "file-included")         { out.wNoFileIncluded       = false; return true; }
     if (str == "no-operation-undefined")   { out.wNoOperationUndefined = true;  return true; }
     if (str ==    "operation-undefined")   { out.wNoOperationUndefined = false; return true; }
-    if (str == "no-variable-unbounded")    { out.wNoVariableUnbounded  = true;  return true; }
-    if (str ==    "variable-unbounded")    { out.wNoVariableUnbounded  = false; return true; }
     if (str == "no-global-variable")       { out.wNoGlobalVariable     = true;  return true; }
     if (str ==    "global-variable")       { out.wNoGlobalVariable     = false; return true; }
     if (str == "no-other")                 { out.wNoOther              = true;  return true; }
@@ -315,6 +341,14 @@ inline bool parseWarning(const std::string& str, GringoOptions& out) {
     return false;
 }
 
+inline bool parsePreserveFacts(const std::string& str, GringoOptions& out) {
+    if (str == "none")   { out.keepFacts = false; out.outputOptions.preserveFacts = false; return true; }
+    if (str == "body")   { out.keepFacts = true;  out.outputOptions.preserveFacts = false; return true; }
+    if (str == "symtab") { out.keepFacts = false; out.outputOptions.preserveFacts = true;  return true; }
+    if (str == "all")    { out.keepFacts = true;  out.outputOptions.preserveFacts = true;  return true; }
+    return false;
+}
+
 static bool parseText(const std::string&, GringoOptions& out) {
     out.outputFormat = Output::OutputFormat::TEXT;
     return true;
@@ -323,7 +357,7 @@ static bool parseText(const std::string&, GringoOptions& out) {
 struct GringoApp : public Potassco::Application {
     using StringSeq = std::vector<std::string>;
     virtual const char* getName() const    { return "gringo"; }
-    virtual const char* getVersion() const { return CLINGO_VERSION_STRING; }
+    virtual const char* getVersion() const { return clingo_version_string(); }
     virtual HelpOpt     getHelpOption() const { return HelpOpt("Print (<n {1=default|2=advanced}) help and exit", 2); }
     virtual void initOptions(Potassco::ProgramOptions::OptionContext& root) {
         using namespace Potassco::ProgramOptions;
@@ -353,20 +387,26 @@ struct GringoApp : public Potassco::Application {
              "      translate: print translated rules as plain text (prefix %%%%)\n"
              "      all      : combines text and translate")
             ("warn,W,@1", storeTo(grOpts_, parseWarning)->arg("<warn>")->composing(), "Enable/disable warnings:\n"
-             "      none:                     disable all warnings\n"
-             "      all:                      enable all warnings\n"
-             "      [no-]atom-undefined:      a :- b.\n"
-             "      [no-]file-included:       #include \"a.lp\". #include \"a.lp\".\n"
+             "      none                    : disable all warnings\n"
+             "      all                     : enable all warnings\n"
+             "      [no-]atom-undefined     : a :- b.\n"
+             "      [no-]file-included      : #include \"a.lp\". #include \"a.lp\".\n"
              "      [no-]operation-undefined: p(1/0).\n"
-             "      [no-]variable-unbounded:  $x > 10.\n"
-             "      [no-]global-variable:     :- #count { X } = 1, X = 1.\n"
-             "      [no-]other:               uncategorized warnings")
+             "      [no-]global-variable    : :- #count { X } = 1, X = 1.\n"
+             "      [no-]other              : uncategorized warnings")
             ("rewrite-minimize,@1", flag(grOpts_.rewriteMinimize = false), "Rewrite minimize constraints into rules")
-            ("keep-facts,@1", flag(grOpts_.keepFacts = false), "Do not remove facts from normal rules")
+            // for backward compatibility
+            ("keep-facts,@4", flag(grOpts_.keepFacts = false), "Preserve facts in rule bodies.")
+            ("preserve-facts,@1", storeTo(grOpts_, parsePreserveFacts),
+             "Preserve facts in output:\n"
+             "      none  : do not preserve\n"
+             "      body  : do not preserve\n"
+             "      symtab: do not preserve\n"
+             "      all   : preserve all facts")
             ("reify-sccs,@1", flag(grOpts_.outputOptions.reifySCCs = false), "Calculate SCCs for reified output")
             ("reify-steps,@1", flag(grOpts_.outputOptions.reifySteps = false), "Add step numbers to reified output")
+            ("show-preds,@1", storeTo(grOpts_.sigvec, parseSigVec), "Show the given signatures")
             ("single-shot,@2", flag(grOpts_.singleShot = false), "Force single-shot grounding mode")
-            ("foobar,@4", storeTo(grOpts_.foobar, parseFoobar), "Foobar")
             ;
         root.add(gringo);
         OptionGroup basic("Basic Options");
@@ -431,8 +471,8 @@ struct GringoApp : public Potassco::Application {
             using namespace Gringo;
             grOpts_.verbose = verbose() == UINT_MAX;
             Output::OutputPredicates outPreds;
-            for (auto &x : grOpts_.foobar) {
-                outPreds.emplace_back(Location("<cmd>",1,1,"<cmd>", 1,1), x, false);
+            for (auto &x : grOpts_.sigvec) {
+                outPreds.add(Location("<cmd>",1,1,"<cmd>", 1,1), x, false);
             }
             Potassco::TheoryData data;
             data.update();
