@@ -22,6 +22,7 @@
 
 // }}}
 
+#include "clingo.hh"
 #include "tests.hh"
 #include <iostream>
 #include <fstream>
@@ -109,6 +110,20 @@ TEST_CASE("solving", "[clingo]") {
                 REQUIRE(messages.empty());
             }
         }
+        SECTION("solve") {
+            ctl.add("base", {}, "x(1).");
+            ctl.ground({{"base", {}}});
+            int n = 0;
+            auto atms = ctl.symbolic_atoms();
+            for (auto it = atms.begin(Signature{"x", 1, false}), ie = atms.end(); it != ie; ++it) {
+                ++n;
+            }
+            REQUIRE(n == 0);
+            for (auto it = atms.begin(Signature{"x", 1, true}), ie = atms.end(); it != ie; ++it) {
+                ++n;
+            }
+            REQUIRE(n == 1);
+        }
         SECTION("get_statistics") {
             ctl.add("pigeon", {"p", "h"}, "1 {p(P,H) : P=1..p}1 :- H=1..h."
                                           "1 {p(P,H) : H=1..h}1 :- P=1..p.");
@@ -288,6 +303,42 @@ TEST_CASE("solving", "[clingo]") {
             REQUIRE(models == (ModelVec{{ a(2), b(1), e(3) }, { a(2), e(3) }, { b(1), e(3) }, { e(3) }}));
 
         }
+        SECTION("backend-theory") {
+            ctl.with_backend([](Backend &backend) {
+                auto num_one = backend.add_theory_term_number(1);
+                auto num_two = backend.add_theory_term_number(2);
+                auto str_x = backend.add_theory_term_string("x");
+                auto fun = backend.add_theory_term_function("f", {num_one, num_two, str_x});
+                auto seq = backend.add_theory_term_sequence(TheorySequenceType::Set, {num_one, num_two, str_x});
+                auto fseq = backend.add_theory_term_function("f", {seq});
+
+                REQUIRE(num_one == backend.add_theory_term_number(1));
+                REQUIRE(num_two == backend.add_theory_term_number(2));
+                REQUIRE(str_x == backend.add_theory_term_string("x"));
+                REQUIRE(num_one == backend.add_theory_term_symbol(parse_term("1")));
+                REQUIRE(num_two == backend.add_theory_term_symbol(parse_term("2")));
+                REQUIRE(fun == backend.add_theory_term_symbol(parse_term("f(1,2,x)")));
+
+                auto elem = backend.add_theory_element({num_one, num_two, seq, fun}, {1, -2, 3});
+
+                backend.theory_atom(0, fun, {});
+                backend.theory_atom(0, backend.add_theory_term_symbol(parse_term("g(1,2)")), {});
+                backend.theory_atom(0, fseq, {elem});
+            });
+            auto atoms = ctl.theory_atoms();
+            using SV = std::vector<S>;
+            SV str_atoms;
+            TheoryAtom last{nullptr, 0};
+            for (auto atom : atoms) {
+                str_atoms.emplace_back(atom.to_string());
+                last = atom;
+            }
+            REQUIRE(str_atoms == SV{
+                "&f(1,2,x){}",
+                "&g(1,2){}",
+                "&f({1,2,x}){1,2,{1,2,x},f(1,2,x): #aux(1),not #aux(2),#aux(3)}"});
+            REQUIRE(last.elements().back().condition() == LiteralSpan{{1, -2, 3}});
+        }
         SECTION("optimize") {
             ctl.add("base", {}, "2 {a; b; c; d}.\n"
                                 ":- a, b.\n"
@@ -297,12 +348,15 @@ TEST_CASE("solving", "[clingo]") {
             ctl.ground({{"base", {}}});
             SymbolVector model;
             CostVector optimum;
-            for (auto &m : ctl.solve()) {
+            PriorityVector priorities;
+            for (auto const &m : ctl.solve()) {
                 model = m.symbols();
                 optimum = m.cost();
+                priorities = m.priorities();
             }
             REQUIRE(model == SymbolVector({ Id("a"), Id("d") }));
             REQUIRE(optimum == (CostVector{7,5}));
+            REQUIRE(priorities == (std::vector<int>{2, 1}));
             REQUIRE(messages.empty());
         }
 #if defined(CLASP_HAS_THREADS) && CLASP_HAS_THREADS == 1
@@ -316,15 +370,14 @@ TEST_CASE("solving", "[clingo]") {
         }
 #endif
         SECTION("model") {
-            ctl.add("base", {}, "a. $x $= 1. #show b.");
+            ctl.add("base", {}, "a. #show b.");
             ctl.ground({{"base", {}}});
             bool sat = false;
             for (auto &m : ctl.solve()) {
                 REQUIRE(m.type() == ModelType::StableModel);
                 REQUIRE(m.symbols(ShowType::Atoms) == (SymbolVector{Id("a")}));
                 REQUIRE(m.symbols(ShowType::Terms) == (SymbolVector{Id("b")}));
-                REQUIRE(m.symbols(ShowType::CSP) == (SymbolVector{Function("$", {Id("x"), Number(1)})}));
-                REQUIRE(m.symbols(ShowType::Shown) == (SymbolVector{Function("$", {Id("x"), Number(1)}), Id("a"), Id("b")}));
+                REQUIRE(m.symbols(ShowType::Shown) == (SymbolVector{Id("a"), Id("b")}));
                 REQUIRE(m.symbols(ShowType::Atoms | ShowType::Complement).size() == 0);
                 REQUIRE( m.contains(Id("a")));
                 REQUIRE(!m.contains(Id("b")));
