@@ -1,37 +1,71 @@
-'''
+"""
 This module contains the `clingo.control.Control` class responsible for
 controling grounding and solving.
 
 Examples
 --------
-The following example shows basic (multishot) grounding and solving.
+The following example shows the basic ground solve process:
 
     >>> from clingo.symbol import Number
     >>> from clingo.control import Control
     >>>
     >>> ctl = Control()
-    >>> ctl.add("base", [], "q.")
-    >>> ctl.add("p", ["t"], "q(t).")
+    >>> ctl.add("q.")
     >>>
-    >>> ctl.ground([("base", [])])
+    >>> ctl.ground()
     >>> print(ctl.solve(on_model=print))
     q
     SAT
-    >>> ctl.ground([("p", [Number(1)]), ("p", [Number(2)])])
+
+The following example shows basic (multishot) grounding and solving:
+
+    >>> from clingo.symbol import Number
+    >>> from clingo.control import Control
+    >>>
+    >>> ctl = Control()
+    >>> ctl.add("a", [], "q.")
+    >>> ctl.add("b", ["t"], "q(t).")
+    >>>
+    >>> ctl.ground([("a", [])])
+    >>> print(ctl.solve(on_model=print))
+    q
+    SAT
+    >>> ctl.ground([("b", [Number(1)]), ("p", [Number(2)])])
     >>> print(ctl.solve(on_model=print))
     q q(1) q(2)
     SAT
-    >>> ctl.ground([("p", [Number(3)])])
+    >>> ctl.ground([("b", [Number(3)])])
     >>> print(ctl.solve(on_model=print))
     q q(1) q(2) q(3)
     SAT
-'''
+"""
 
-from typing import Any, Callable, Iterator, Optional, Sequence, Tuple, Union, cast
-from collections import abc
 import sys
+from collections import abc
+from typing import (
+    Any,
+    Callable,
+    Iterator,
+    Optional,
+    Sequence,
+    Tuple,
+    Union,
+    cast,
+    overload,
+)
+if sys.version_info >= (3, 8):
+    from typing import Literal
 
-from ._internal import _CBData, _Error, _cb_error_handler, _c_call, _ffi, _handle_error, _lib, _overwritten
+from ._internal import (
+    _CBData,
+    _Error,
+    _cb_error_handler,
+    _c_call,
+    _ffi,
+    _handle_error,
+    _lib,
+    _overwritten,
+)
 from .core import Logger
 from .symbol import Symbol
 from .symbolic_atoms import SymbolicAtoms
@@ -42,7 +76,8 @@ from .backend import Backend, Observer
 from .configuration import Configuration
 from .statistics import StatisticsMap, _mutable_statistics, _statistics
 
-__all__ = [ 'Control' ]
+__all__ = ["Control"]
+
 
 class _SolveEventHandler:
     # pylint: disable=missing-function-docstring
@@ -70,40 +105,52 @@ class _SolveEventHandler:
         if self._on_statistics is not None:
             self._on_statistics(_mutable_statistics(step), _mutable_statistics(accu))
 
-@_ffi.def_extern(onerror=_cb_error_handler('data'), name='pyclingo_solve_event_callback')
+
+@_ffi.def_extern(
+    onerror=_cb_error_handler("data"), name="pyclingo_solve_event_callback"
+)
 def _pyclingo_solve_event_callback(type_, event, data, goon):
-    '''
+    """
     Low-level solve event handler.
-    '''
+    """
     handler = _ffi.from_handle(data).data
     if type_ == _lib.clingo_solve_event_type_finish:
-        handler.on_finish(_ffi.cast('clingo_solve_result_bitset_t*', event)[0])
+        handler.on_finish(_ffi.cast("clingo_solve_result_bitset_t*", event)[0])
 
     if type_ == _lib.clingo_solve_event_type_model:
-        goon[0] = handler.on_model(_ffi.cast('clingo_model_t*', event))
+        goon[0] = handler.on_model(_ffi.cast("clingo_model_t*", event))
 
     if type_ == _lib.clingo_solve_event_type_unsat:
-        c_args = _ffi.cast('void**', event)
-        c_lower = _ffi.cast('int64_t*', c_args[0])
-        size = int(_ffi.cast('size_t', c_args[1]))
+        c_args = _ffi.cast("void**", event)
+        c_lower = _ffi.cast("int64_t*", c_args[0])
+        size = int(_ffi.cast("size_t", c_args[1]))
         handler.on_unsat([c_lower[i] for i in range(size)])
 
     if type_ == _lib.clingo_solve_event_type_statistics:
-        p_stats = _ffi.cast('clingo_statistics_t**', event)
+        p_stats = _ffi.cast("clingo_statistics_t**", event)
         handler.on_statistics(p_stats[0], p_stats[1])
 
     return True
 
-@_ffi.def_extern(onerror=_cb_error_handler('data'), name='pyclingo_ground_callback')
-def _pyclingo_ground_callback(location, name, arguments, arguments_size, data, symbol_callback, symbol_callback_data):
-    '''
+
+@_ffi.def_extern(onerror=_cb_error_handler("data"), name="pyclingo_ground_callback")
+def _pyclingo_ground_callback(
+    location,
+    name,
+    arguments,
+    arguments_size,
+    data,
+    symbol_callback,
+    symbol_callback_data,
+):
+    """
     Low-level ground callback.
-    '''
+    """
     # Note: location could be attached to error message
     # pylint: disable=protected-access,unused-argument
     context = _ffi.from_handle(data).data
     py_name = _ffi.string(name).decode()
-    fun = getattr(sys.modules['__main__'] if context is None else context, py_name)
+    fun = getattr(sys.modules["__main__"] if context is None else context, py_name)
 
     args = []
     for i in range(arguments_size):
@@ -112,15 +159,16 @@ def _pyclingo_ground_callback(location, name, arguments, arguments_size, data, s
     ret = fun(*args)
     symbols = list(ret) if isinstance(ret, abc.Iterable) else [ret]
 
-    c_symbols = _ffi.new('clingo_symbol_t[]', len(symbols))
+    c_symbols = _ffi.new("clingo_symbol_t[]", len(symbols))
     for i, sym in enumerate(symbols):
         c_symbols[i] = sym._rep
     _handle_error(symbol_callback(c_symbols, len(symbols), symbol_callback_data))
 
     return True
 
+
 class Control:
-    '''
+    """
     Control object for the grounding/solving process.
 
     Parameters
@@ -137,9 +185,14 @@ class Control:
     Note that only gringo options (without `--text`) and clasp's search options
     are supported. Furthermore, you must not call any functions of a `Control`
     object while a solve call is active.
-    '''
-    def __init__(self, arguments: Sequence[str]=[],
-                 logger: Optional[Logger]=None, message_limit: int=20):
+    """
+
+    def __init__(
+        self,
+        arguments: Sequence[str] = [],
+        logger: Optional[Logger] = None,
+        message_limit: int = 20,
+    ):
         # pylint: disable=protected-access,dangerous-default-value
         self._free = False
         self._mem = []
@@ -152,12 +205,19 @@ class Control:
                 c_handle = _ffi.NULL
                 c_cb = _ffi.NULL
             c_mem = []
-            c_args = _ffi.new('char*[]', len(arguments))
+            c_args = _ffi.new("char*[]", len(arguments))
             for i, arg in enumerate(arguments):
                 c_mem.append(_ffi.new("char[]", arg.encode()))
                 c_args[i] = c_mem[-1]
-            self._rep = _c_call('clingo_control_t *', _lib.clingo_control_new,
-                                c_args, len(arguments), c_cb, c_handle, message_limit)
+            self._rep = _c_call(
+                "clingo_control_t *",
+                _lib.clingo_control_new,
+                c_args,
+                len(arguments),
+                c_cb,
+                c_handle,
+                message_limit,
+            )
             self._free = True
         else:
             self._rep = arguments
@@ -171,8 +231,9 @@ class Control:
         if self._free:
             _lib.clingo_control_free(self._rep)
 
+    @overload
     def add(self, name: str, parameters: Sequence[str], program: str) -> None:
-        '''
+        """
         Extend the logic program with the given non-ground logic program in string form.
 
         Parameters
@@ -187,22 +248,81 @@ class Control:
         See Also
         --------
         Control.ground
-        '''
+        """
+
+    @overload
+    def add(self, program: str) -> None:
+        """
+        Extend the logic program with the given non-ground logic program in string form.
+
+        Parameters
+        ----------
+        name
+            The name of program block to add.
+
+        Notes
+        -----
+        This function is equivalent to calling `add("base", [], program)`.
+        """
+
+    def add(self, *args, **kwargs) -> None:
+        """
+        Extend the logic program with the given non-ground logic program in string form.
+
+        This function provides two overloads:
+
+        ```python
+        def add(self, name: str, parameters: Sequence[str], program: str) -> None:
+            ...
+
+        def add(self, program: str) -> None:
+            return self.add("base", [], program)
+        ```
+
+        Parameters
+        ----------
+        name
+            The name of program block to add.
+        parameters
+            The parameters of the program block to add.
+        program
+            The non-ground program in string form.
+
+        See Also
+        --------
+        Control.ground
+        """
+        n = len(args) + len(kwargs)
+        if n == 1:
+            self._add1(*args, **kwargs)
+        else:
+            self._add2(*args, **kwargs)
+
+    def _add1(self, program: str) -> None:
+        self._add2("base", [], program)
+
+    def _add2(self, name: str, parameters: Sequence[str], program: str) -> None:
         c_mem = []
-        c_params = _ffi.new('char*[]', len(parameters))
+        c_params = _ffi.new("char*[]", len(parameters))
         for i, param in enumerate(parameters):
             c_mem.append(_ffi.new("char[]", param.encode()))
             c_params[i] = c_mem[-1]
-        _handle_error(_lib.clingo_control_add(self._rep, name.encode(), c_params, len(parameters), program.encode()))
+        _handle_error(
+            _lib.clingo_control_add(
+                self._rep, name.encode(), c_params, len(parameters), program.encode()
+            )
+        )
 
-    def _program_atom(self, lit: Union[Symbol,int]) -> int:
+    def _program_atom(self, lit: Union[Symbol, int]) -> int:
         if isinstance(lit, int):
             return lit
         satom = self.symbolic_atoms[lit]
         return 0 if satom is None else satom.literal
 
-    def assign_external(self, external: Union[Symbol,int], truth: Optional[bool]) -> None:
-        '''
+    def assign_external(
+        self, external: Union[Symbol, int], truth: Optional[bool]
+    ) -> None:
+        """
         Assign a truth value to an external atom.
 
         Parameters
@@ -229,7 +349,7 @@ class Control:
 
         For convenience, the truth assigned to atoms over negative program
         literals is inverted.
-        '''
+        """
 
         if truth is None:
             val = _lib.clingo_external_type_free
@@ -237,21 +357,28 @@ class Control:
             val = _lib.clingo_external_type_true
         else:
             val = _lib.clingo_external_type_false
-        _handle_error(_lib.clingo_control_assign_external(self._rep, self._program_atom(external), val))
+        _handle_error(
+            _lib.clingo_control_assign_external(
+                self._rep, self._program_atom(external), val
+            )
+        )
 
     def backend(self) -> Backend:
-        '''
+        """
         Returns a `Backend` object providing a low level interface to extend a
         logic program.
 
         See Also
         --------
         clingo.backend
-        '''
-        return Backend(_c_call('clingo_backend_t*', _lib.clingo_control_backend, self._rep), self._error)
+        """
+        return Backend(
+            _c_call("clingo_backend_t*", _lib.clingo_control_backend, self._rep),
+            self._error,
+        )
 
     def cleanup(self) -> None:
-        '''
+        """
         Cleanup the domain used for grounding by incorporating information from
         the solver.
 
@@ -276,11 +403,11 @@ class Control:
 
         Typically, it is not necessary to call this function manually because
         automatic cleanups are enabled by default.
-        '''
+        """
         _handle_error(_lib.clingo_control_cleanup(self._rep))
 
     def get_const(self, name: str) -> Optional[Symbol]:
-        '''
+        """
         Return the symbol for a constant definition of form:
 
             #const name = symbol.
@@ -293,14 +420,25 @@ class Control:
         Returns
         -------
         The function returns `None` if no matching constant definition exists.
-        '''
-        if not _c_call('bool', _lib.clingo_control_has_const, self._rep, name.encode()):
+        """
+        if not _c_call("bool", _lib.clingo_control_has_const, self._rep, name.encode()):
             return None
 
-        return Symbol(_c_call('clingo_symbol_t', _lib.clingo_control_get_const, self._rep, name.encode()))
+        return Symbol(
+            _c_call(
+                "clingo_symbol_t",
+                _lib.clingo_control_get_const,
+                self._rep,
+                name.encode(),
+            )
+        )
 
-    def ground(self, parts: Sequence[Tuple[str,Sequence[Symbol]]], context: Any=None) -> None:
-        '''
+    def ground(
+        self,
+        parts: Sequence[Tuple[str, Sequence[Symbol]]] = (("base", ()),),
+        context: Any = None,
+    ) -> None:
+        """
         Ground the given list of program parts specified by tuples of names and
         arguments.
 
@@ -317,7 +455,7 @@ class Control:
         Note that parts of a logic program without an explicit `#program`
         specification are by default put into a program called `base` without
         arguments.
-        '''
+        """
         # pylint: disable=protected-access,dangerous-default-value
         self._error.clear()
         data = _CBData(context, self._error)
@@ -335,11 +473,13 @@ class Control:
                 c_part.params[i] = sym._rep
             c_part.size = len(part[1])
 
-        _handle_error(_lib.clingo_control_ground(
-            self._rep, c_parts, len(parts), c_cb, c_data), data)
+        _handle_error(
+            _lib.clingo_control_ground(self._rep, c_parts, len(parts), c_cb, c_data),
+            data,
+        )
 
     def interrupt(self) -> None:
-        '''
+        """
         Interrupt the active solve call.
 
         Notes
@@ -348,22 +488,22 @@ class Control:
         search is active, the subsequent call to `Control.solve` is interrupted. The
         result of the `Control.solve` method can be used to query if the search was
         interrupted.
-        '''
+        """
         _lib.clingo_control_interrupt(self._rep)
 
     def load(self, path: str) -> None:
-        '''
+        """
         Extend the logic program with a (non-ground) logic program in a file.
 
         Parameters
         ----------
         path
             The path of the file to load.
-        '''
+        """
         _handle_error(_lib.clingo_control_load(self._rep, path.encode()))
 
-    def register_observer(self, observer: Observer, replace: bool=False) -> None:
-        '''
+    def register_observer(self, observer: Observer, replace: bool = False) -> None:
+        """
         Registers the given observer to inspect the produced grounding.
 
         Parameters
@@ -378,35 +518,80 @@ class Control:
         See Also
         --------
         clingo.backend
-        '''
+        """
         # pylint: disable=protected-access,line-too-long
-        c_observer = _ffi.new('clingo_ground_program_observer_t*', (
-            _lib.pyclingo_observer_init_program if _overwritten(Observer, observer, "init_program") else _ffi.NULL,
-            _lib.pyclingo_observer_begin_step if _overwritten(Observer, observer, "begin_step") else _ffi.NULL,
-            _lib.pyclingo_observer_end_step if _overwritten(Observer, observer, "end_step") else _ffi.NULL,
-            _lib.pyclingo_observer_rule if _overwritten(Observer, observer, "rule") else _ffi.NULL,
-            _lib.pyclingo_observer_weight_rule if _overwritten(Observer, observer, "weight_rule") else _ffi.NULL,
-            _lib.pyclingo_observer_minimize if _overwritten(Observer, observer, "minimize") else _ffi.NULL,
-            _lib.pyclingo_observer_project if _overwritten(Observer, observer, "project") else _ffi.NULL,
-            _lib.pyclingo_observer_output_atom if _overwritten(Observer, observer, "output_atom") else _ffi.NULL,
-            _lib.pyclingo_observer_output_term if _overwritten(Observer, observer, "output_term") else _ffi.NULL,
-            _lib.pyclingo_observer_output_csp if _overwritten(Observer, observer, "output_csp") else _ffi.NULL,
-            _lib.pyclingo_observer_external if _overwritten(Observer, observer, "external") else _ffi.NULL,
-            _lib.pyclingo_observer_assume if _overwritten(Observer, observer, "assume") else _ffi.NULL,
-            _lib.pyclingo_observer_heuristic if _overwritten(Observer, observer, "heuristic") else _ffi.NULL,
-            _lib.pyclingo_observer_acyc_edge if _overwritten(Observer, observer, "acyc_edge") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_term_number if _overwritten(Observer, observer, "theory_term_number") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_term_string if _overwritten(Observer, observer, "theory_term_string") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_term_compound if _overwritten(Observer, observer, "theory_term_compound") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_element if _overwritten(Observer, observer, "theory_element") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_atom if _overwritten(Observer, observer, "theory_atom") else _ffi.NULL,
-            _lib.pyclingo_observer_theory_atom_with_guard if _overwritten(Observer, observer, "theory_atom_with_guard") else _ffi.NULL))
+        c_observer = _ffi.new(
+            "clingo_ground_program_observer_t*",
+            (
+                _lib.pyclingo_observer_init_program
+                if _overwritten(Observer, observer, "init_program")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_begin_step
+                if _overwritten(Observer, observer, "begin_step")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_end_step
+                if _overwritten(Observer, observer, "end_step")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_rule
+                if _overwritten(Observer, observer, "rule")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_weight_rule
+                if _overwritten(Observer, observer, "weight_rule")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_minimize
+                if _overwritten(Observer, observer, "minimize")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_project
+                if _overwritten(Observer, observer, "project")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_output_atom
+                if _overwritten(Observer, observer, "output_atom")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_output_term
+                if _overwritten(Observer, observer, "output_term")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_external
+                if _overwritten(Observer, observer, "external")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_assume
+                if _overwritten(Observer, observer, "assume")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_heuristic
+                if _overwritten(Observer, observer, "heuristic")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_acyc_edge
+                if _overwritten(Observer, observer, "acyc_edge")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_term_number
+                if _overwritten(Observer, observer, "theory_term_number")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_term_string
+                if _overwritten(Observer, observer, "theory_term_string")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_term_compound
+                if _overwritten(Observer, observer, "theory_term_compound")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_element
+                if _overwritten(Observer, observer, "theory_element")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_atom
+                if _overwritten(Observer, observer, "theory_atom")
+                else _ffi.NULL,
+                _lib.pyclingo_observer_theory_atom_with_guard
+                if _overwritten(Observer, observer, "theory_atom_with_guard")
+                else _ffi.NULL,
+            ),
+        )
         c_data = _ffi.new_handle(_CBData(observer, self._error))
         self._mem.append(c_data)
-        _handle_error(_lib.clingo_control_register_observer(self._rep, c_observer, replace, c_data))
+        _handle_error(
+            _lib.clingo_control_register_observer(
+                self._rep, c_observer, replace, c_data
+            )
+        )
 
     def register_propagator(self, propagator: Propagator) -> None:
-        '''
+        """
         Registers the given propagator with all solvers.
 
         Parameters
@@ -417,20 +602,38 @@ class Control:
         See Also
         --------
         clingo.propagator
-        '''
+        """
         # pylint: disable=protected-access
-        c_propagator = _ffi.new('clingo_propagator_t*', (
-            _lib.pyclingo_propagator_init if _overwritten(Propagator, propagator, "init") else _ffi.NULL,
-            _lib.pyclingo_propagator_propagate if _overwritten(Propagator, propagator, "propagate") else _ffi.NULL,
-            _lib.pyclingo_propagator_undo if _overwritten(Propagator, propagator, "undo") else _ffi.NULL,
-            _lib.pyclingo_propagator_check if _overwritten(Propagator, propagator, "check") else _ffi.NULL,
-            _lib.pyclingo_propagator_decide if _overwritten(Propagator, propagator, "decide") else _ffi.NULL))
+        c_propagator = _ffi.new(
+            "clingo_propagator_t*",
+            (
+                _lib.pyclingo_propagator_init
+                if _overwritten(Propagator, propagator, "init")
+                else _ffi.NULL,
+                _lib.pyclingo_propagator_propagate
+                if _overwritten(Propagator, propagator, "propagate")
+                else _ffi.NULL,
+                _lib.pyclingo_propagator_undo
+                if _overwritten(Propagator, propagator, "undo")
+                else _ffi.NULL,
+                _lib.pyclingo_propagator_check
+                if _overwritten(Propagator, propagator, "check")
+                else _ffi.NULL,
+                _lib.pyclingo_propagator_decide
+                if _overwritten(Propagator, propagator, "decide")
+                else _ffi.NULL,
+            ),
+        )
         c_data = _ffi.new_handle(_CBData(propagator, self._error))
         self._mem.append(c_data)
-        _handle_error(_lib.clingo_control_register_propagator(self._rep, c_propagator, c_data, False))
+        _handle_error(
+            _lib.clingo_control_register_propagator(
+                self._rep, c_propagator, c_data, False
+            )
+        )
 
-    def release_external(self, external: Union[Symbol,int]) -> None:
-        '''
+    def release_external(self, external: Union[Symbol, int]) -> None:
+        """
         Release an external atom represented by the given symbol or program
         literal.
 
@@ -466,19 +669,92 @@ class Control:
             >>> print(ctl.solve(on_model=print))
             a
             SAT
-        '''
-        _handle_error(_lib.clingo_control_release_external(self._rep, self._program_atom(external)))
+        """
+        _handle_error(
+            _lib.clingo_control_release_external(
+                self._rep, self._program_atom(external)
+            )
+        )
 
-    def solve(self,
-              assumptions: Sequence[Union[Tuple[Symbol,bool],int]]=[],
-              on_model: Callable[[Model],Optional[bool]]=None,
-              on_unsat: Callable[[Sequence[int]],None]=None,
-              on_statistics : Callable[[StatisticsMap,StatisticsMap],None]=None,
-              on_finish: Callable[[SolveResult],None]=None,
-              on_core: Callable[[Sequence[int]],None]=None,
-              yield_: bool=False,
-              async_: bool=False) -> Union[SolveHandle,SolveResult]:
-        '''
+    # this assumes that overloads are matched in the order they appear
+    # unfortunately, PEP0484 does not specify any kind of semantics just giving examples
+    if sys.version_info >= (3, 8):
+        @overload
+        def solve(
+            self,
+            assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+            on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+            on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+            on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+            on_finish: Optional[Callable[[SolveResult], None]] = None,
+            on_core: Optional[Callable[[Sequence[int]], None]] = None,
+            *,
+            yield_: Literal[False] = False,
+            async_: Literal[False] = False,
+        ) -> SolveResult: ...
+        @overload
+        def solve(
+            self,
+            assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+            on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+            on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+            on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+            on_finish: Optional[Callable[[SolveResult], None]] = None,
+            on_core: Optional[Callable[[Sequence[int]], None]] = None,
+            *,
+            yield_: Literal[True],
+            async_: bool = False,
+        ) -> SolveHandle: ...
+        @overload
+        def solve(
+            self,
+            assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+            on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+            on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+            on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+            on_finish: Optional[Callable[[SolveResult], None]] = None,
+            on_core: Optional[Callable[[Sequence[int]], None]] = None,
+            *,
+            yield_: bool = False,
+            async_: Literal[True],
+        ) -> SolveHandle: ...
+    else:
+        @overload
+        def solve(
+            self,
+            assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+            on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+            on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+            on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+            on_finish: Optional[Callable[[SolveResult], None]] = None,
+            on_core: Optional[Callable[[Sequence[int]], None]] = None,
+        ) -> SolveResult: ...
+
+    @overload
+    def solve(
+        self,
+        assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+        on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+        on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+        on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+        on_finish: Optional[Callable[[SolveResult], None]] = None,
+        on_core: Optional[Callable[[Sequence[int]], None]] = None,
+        yield_: bool = False,
+        async_: bool = False,
+    ) -> Union[SolveHandle, SolveResult]: ...
+
+    def solve(
+        self,
+        assumptions: Sequence[Union[Tuple[Symbol, bool], int]] = (),
+        on_model: Optional[Callable[[Model], Optional[bool]]] = None,
+        on_unsat: Optional[Callable[[Sequence[int]], None]] = None,
+        on_statistics: Optional[Callable[[StatisticsMap, StatisticsMap], None]] = None,
+        on_finish: Optional[Callable[[SolveResult], None]] = None,
+        on_core: Optional[Callable[[Sequence[int]], None]] = None,
+        yield_: bool = False,
+        async_: bool = False,
+    ) -> Union[SolveHandle, SolveResult]:
+        """
         Starts a search.
 
         Parameters
@@ -541,7 +817,7 @@ class Control:
 
         This function as well as blocking functions on the
         `clingo.solving.SolveHandle` release the GIL but are not thread-safe.
-        '''
+        """
         # pylint: disable=protected-access,dangerous-default-value
         self._error.clear()
         handler = _SolveEventHandler(on_model, on_unsat, on_statistics, on_finish)
@@ -551,7 +827,7 @@ class Control:
         p_ass = _ffi.NULL
         if assumptions:
             atoms = None
-            p_ass = _ffi.new('clingo_literal_t[]', len(assumptions))
+            p_ass = _ffi.new("clingo_literal_t[]", len(assumptions))
             for i, lit in enumerate(assumptions):
                 if isinstance(lit, int):
                     p_ass[i] = lit
@@ -569,12 +845,19 @@ class Control:
             mode |= _lib.clingo_solve_mode_async
 
         handle = SolveHandle(
-            _c_call('clingo_solve_handle_t*', _lib.clingo_control_solve,
-                self._rep, mode,
-                p_ass, len(assumptions),
-                _lib.pyclingo_solve_event_callback, self._handler,
-                handler=data),
-            handler)
+            _c_call(
+                "clingo_solve_handle_t*",
+                _lib.clingo_control_solve,
+                self._rep,
+                mode,
+                p_ass,
+                len(assumptions),
+                _lib.pyclingo_solve_event_callback,
+                self._handler,
+                handler=data,
+            ),
+            data,
+        )
 
         if not yield_ and not async_:
             with handle:
@@ -586,18 +869,20 @@ class Control:
 
     @property
     def configuration(self) -> Configuration:
-        '''
+        """
         Object to change the configuration.
-        '''
-        conf = _c_call('clingo_configuration_t*', _lib.clingo_control_configuration, self._rep)
-        key = _c_call('clingo_id_t', _lib.clingo_configuration_root, conf)
+        """
+        conf = _c_call(
+            "clingo_configuration_t*", _lib.clingo_control_configuration, self._rep
+        )
+        key = _c_call("clingo_id_t", _lib.clingo_configuration_root, conf)
         return Configuration(conf, key)
 
     @property
     def enable_cleanup(self) -> bool:
-        '''
+        """
         Whether to enable automatic calls to `Control.cleanup`.
-        '''
+        """
         return _lib.clingo_control_get_enable_cleanup(self._rep)
 
     @enable_cleanup.setter
@@ -606,7 +891,7 @@ class Control:
 
     @property
     def enable_enumeration_assumption(self) -> bool:
-        '''
+        """
         Whether do discard or keep learnt information from enumeration modes.
 
         If the enumeration assumption is enabled, then all information learnt from
@@ -624,16 +909,18 @@ class Control:
         unpredictable because there are no guarantees which information exactly is
         kept. There might be small speed benefits when disabling the enumeration
         assumption for single shot solving.
-        '''
+        """
         return _lib.clingo_control_get_enable_enumeration_assumption(self._rep)
 
     @enable_enumeration_assumption.setter
     def enable_enumeration_assumption(self, value: bool) -> None:
-        _handle_error(_lib.clingo_control_set_enable_enumeration_assumption(self._rep, value))
+        _handle_error(
+            _lib.clingo_control_set_enable_enumeration_assumption(self._rep, value)
+        )
 
     @property
     def is_conflicting(self) -> bool:
-        '''
+        """
         Whether the internal program representation is conflicting.
 
         If this (read-only) property is true, solve calls return immediately with an
@@ -644,12 +931,12 @@ class Control:
         Conflicts first have to be detected, e.g., initial unit propagation results in
         an empty clause, or later if an empty clause is resolved during solving. Hence,
         the property might be false even if the problem is unsatisfiable.
-        '''
+        """
         return _lib.clingo_control_is_conflicting(self._rep)
 
     @property
     def statistics(self) -> dict:
-        '''
+        """
         A `dict` containing solve statistics of the last solve call.
 
         See Also
@@ -665,15 +952,21 @@ class Control:
         statistics right after solving.
 
         This property is only available in clingo.
-        '''
-        stats = _c_call('clingo_statistics_t*', _lib.clingo_control_statistics, self._rep)
+        """
+        stats = _c_call(
+            "clingo_statistics_t*", _lib.clingo_control_statistics, self._rep
+        )
 
-        p_key = _ffi.new('uint64_t*')
+        p_key = _ffi.new("uint64_t*")
         key_root = _c_call(p_key, _lib.clingo_statistics_root, stats)
 
-        key_summary = _c_call(p_key, _lib.clingo_statistics_map_at, stats, key_root, "summary".encode())
-        key_call = _c_call(p_key, _lib.clingo_statistics_map_at, stats, key_summary, "call".encode())
-        call = _c_call('double', _lib.clingo_statistics_value_get, stats, key_call)
+        key_summary = _c_call(
+            p_key, _lib.clingo_statistics_map_at, stats, key_root, "summary".encode()
+        )
+        key_call = _c_call(
+            p_key, _lib.clingo_statistics_map_at, stats, key_summary, "call".encode()
+        )
+        call = _c_call("double", _lib.clingo_statistics_value_get, stats, key_call)
         if self._statistics is not None and call != self._statistics_call:
             self._statistics = None
 
@@ -685,26 +978,34 @@ class Control:
 
     @property
     def symbolic_atoms(self) -> SymbolicAtoms:
-        '''
+        """
         An object to inspect the symbolic atoms.
 
         See Also
         --------
         clingo.symbolic_atoms
-        '''
-        return SymbolicAtoms(_c_call('clingo_symbolic_atoms_t*', _lib.clingo_control_symbolic_atoms, self._rep))
+        """
+        return SymbolicAtoms(
+            _c_call(
+                "clingo_symbolic_atoms_t*",
+                _lib.clingo_control_symbolic_atoms,
+                self._rep,
+            )
+        )
 
     @property
     def theory_atoms(self) -> Iterator[TheoryAtom]:
-        '''
+        """
         An iterator over the theory atoms in a program.
 
         See Also
         --------
         clingo.theory_atoms
-        '''
-        atoms = _c_call('clingo_theory_atoms_t*', _lib.clingo_control_theory_atoms, self._rep)
-        size = _c_call('size_t', _lib.clingo_theory_atoms_size, atoms)
+        """
+        atoms = _c_call(
+            "clingo_theory_atoms_t*", _lib.clingo_control_theory_atoms, self._rep
+        )
+        size = _c_call("size_t", _lib.clingo_theory_atoms_size, atoms)
 
         for idx in range(size):
             yield TheoryAtom(atoms, idx)
